#include "Timer.h"
#include "kIndustryCalendar.h"
#include "WorkScheduler.h"
#include "FallingButton.h"
 
////////////////////////////////////////////////////////////////// Phần bổ sung cho ví dụ
const byte startButtonPin = 6;
const byte pauseButtonPin = 7;
const byte stopButtonPin = 8;
FallingButton startButton = FallingButton(startButtonPin), 
              pauseButton = FallingButton(pauseButtonPin),
              stopButton  = FallingButton(stopButtonPin);
 
////////////////////////////////////////////////////////////////// Phần chính
////////////// các job của tiến trình công nghiệp
//job1
void job1() {
  unsigned long time = millis();
  Serial.print("Job1: ");
  Serial.println(time);
}
 
//job2
void job2() {
  unsigned long time = millis();
  Serial.print("Job2: ");
  Serial.println(time);
}
 
void job3() {
  Serial.println("Ban da nhan phim tren Serial");
}
 
bool waitForSerial() {
  bool res = Serial.available();
  //xóa hết buffer của Serial
  while (Serial.available())
    Serial.read();
  return res;
}
 
 
///////////////////////các workscheduler
WorkScheduler *processWorkScheduler;        //xử lý tiến trình công nghiệp
WorkScheduler *buttonWorkScheduler;         //kiểm tra tiến trình button
 
 
/////////////////////// các work
void updateProcessWork() {
  kIndustryCalendar::getInstance()->update();
}
 
void buttonWork() {
  if (startButton.check() && !kIndustryCalendar::getInstance()->isRunning()) { //cho quy trình chạy lại nếu nhán nút khởi động
    kIndustryCalendar::getInstance()->startAgain();
    Serial.println("Start");
  }
 
  if (kIndustryCalendar::getInstance()->isRunning()) {
    if (pauseButton.check()) {                                                 // nhấn nút tạm dừng
      kIndustryCalendar::getInstance()->pause();    
      Serial.println("pause");
    } else if (stopButton.check()) {                                           // nhấn nút thoát
      kIndustryCalendar::getInstance()->stop();
      Serial.println("stop");
    }
  }
}
 
void setup()
{
 
  ///// Phần chính trong thư viện
 
  
  //Khởi tạo serial ở mức baudrate 115200
  Serial.begin(115200); 
 
  
  //Khởi gạo class timer (design pattern singleton) - bắt buộc phải có trong hàm setup (trước khi khởi tạo các job)
  Timer::getInstance()->initialize();
 
  ////////////////// Khởi tạo kIndustryCalendar trước //////////////////
  
 
  //Khởi tạo lịch, các công việc phải được sắp xếp theo chiều thời gian tăng dần để tránh lỗi
  kIndustryCalendar::getInstance()->initialize();
  //Chạy lệnh job1 ở thời điểm 0 trong chu kỳ
  kIndustryCalendar::getInstance()->addJob(job1, (unsigned long)0);
  //Chạy lệnh job2 ở thời điểm 1000ms trong chu kỳ
  kIndustryCalendar::getInstance()->addJob(job2, (unsigned long)1000);
 
  //Job3 sẽ chỉ được thực hiện khi bạn gửi một ký tự gì đó qua Serial (bật serial monitor lên và nhấn bất kì rồi ấn Enter :D)
  kIndustryCalendar::getInstance()->addJob(job3, waitForSerial);
 
 
  //***Gán số lần hoạt động của chu trình
  kIndustryCalendar::getInstance()->setProcessCount(3);
  
 
  ////////////////// Khởi tạo WorkScheduler //////////////////
  //khởi tạo workscheduler cập nhập tiến trình công nghiệp
  processWorkScheduler = new WorkScheduler(0UL, updateProcessWork);
  
  //khởi tạo workscheduler cập nhập giá trị button để quản trị tiến trình công nghiệp
  buttonWorkScheduler = new WorkScheduler(0UL, buttonWork);
}
 
//trong hàm loop chỉ nên có những hàm này, bạn muốn viết một chức năng khác? Xin hãy tạo một job và đưa vào thời khóa biểu scheduler như hàm dưới
 
void loop()
{
  //đầu hàm loop phải có để cập nhập thời điểm diễn ra việc kiểm tra lại các tiến trình
  Timer::getInstance()->update();
 
  //cập nhập tiến trình công nghiệp
  processWorkScheduler->update();
  //cập nhập tiến trình kiểm tra nút nhấn
  buttonWorkScheduler->update();
 
  //cuối hàm loop phải có để cập nhập lại THỜI ĐIỂM (thời điểm chứ ko phải thời gian nha, tuy tiếng Anh chúng đều là time) để cho lần xử lý sau
  Timer::getInstance()->resetTick();
  
}
